K8S_API_VERSION_V1 = "v1"
K8S_API_VERSION_NETWORKING_V1_BETA1 = "networking.k8s.io/v1beta1"
K8S_API_VERSION_BATCH_V1 = "batch/v1"
K8S_API_VERSION_APPS_V1 = "apps/v1"
K8S_PERSISTENT_VOLUME_KIND = "PersistentVolume"
K8S_PERSISTENT_VOLUME_CLAIM_KIND = "PersistentVolumeClaim"
K8S_CONFIG_MAP_KIND = "ConfigMap"
K8S_SECRET_KIND = "Secret"  # noqa
K8S_POD_KIND = "Pod"
K8S_POD_TEMPLATE_KIND = "PodTemplate"
K8S_DEPLOYMENT_KIND = "Deployment"
K8S_SERVICE_KIND = "Service"
K8S_INGRESS_KIND = "Ingress"
K8S_JOB_KIND = "Job"
